skipQueueApp.controller('AddAddressCtrl', function ($scope, $rootScope, $stateParams, UserService, GlobalUser, 
                                                    Utilities, CommonService, $q, Validation) {
    var addressIndex = $stateParams.index;
    $scope.pageName = $stateParams.pageName;
    $scope.index = $stateParams.index;
    
    var userData = null;
    $scope.userAddressList = [];
    var addressDefault = {country:constant.country_default, 
                            countries: $scope.countries,
                            state: constant.state_default, 
                            states:[],
                            city: constant.city_default, 
                            cities:[], 
                            address:"",
                            block_code:"", 
                            zip_code:"", 
                            type:constant.language.plh_type,
                            types:constCommonObj.arr_types,
                            id: "-1",
                            has_change: "0",
                            is_delete: "0"};
    $scope.init = function () {
        $rootScope.$broadcast('loading:show');
        CommonService.getCountry().then(function (data) {
            // Parse data
            $rootScope.$broadcast('loading:hide');
            $scope.countries = angular.copy(data);
            if($scope.pageName === "delivery"){
                _initAddress().then(function(addressData){
                    $scope.address = angular.copy(addressData);
                    if($scope.index == ""){
                        $scope.address.type = {'name': constant.language.plh_type, 'id': -1};
                    }
                });
            }
            else{
                
                initNewAddress().then(function(addressData){
                    $scope.address = angular.copy(addressData);
                    $scope.address.type = {'name': constant.language.plh_type, 'id': -1};
                });
            }
        }, function (error) {
            $rootScope.$broadcast('loading:hide');
            $scope.errorHandler(error);
        });
    };
    
    $scope.init();
    $scope.loadState = function (address) {
        CommonService.getState(address.country.id).then(function (data) {
            $scope.address.states = data;
            $scope.address.state = {'name': constant.language.plh_state, 'id': -1};
            $scope.address.city = {'name': constant.language.plh_city, 'id': -1};
            $scope.address.cities = [];
            $rootScope.$broadcast('loading:hide');
        }, function (error) {
            $rootScope.$broadcast('loading:hide');
            $scope.errorHandler(error);
        });
    };

    $scope.loadCity = function (address) {
        CommonService.getCity(address.state.id).then(function (data) {
            $scope.address.cities = angular.copy(data);
            $scope.address.city = {'name': constant.language.plh_city, 'id': -1};
            $rootScope.$broadcast('loading:hide');
        }, function (error) {
            $rootScope.$broadcast('loading:hide');
            $scope.errorHandler(error);
        });
    };

    $scope.addUpdateAddressClick = function () {
        //validation Address 
        var validAddress = Validation.validationAddress($scope.address);
        if(!validAddress){
            return;
        }
        var validationZipcode = Validation.validationZipCode($scope.address.zip_code);
        if(!validationZipcode.result){
            Utilities.showAlertError(constant.language.mess_title_error, validationZipcode.message);
            return;
        }
        
        // parse data model view -> data model
        var addressModel = UserService.parseAddressModel($scope.address);
        
        // check address 
        if($scope.address.id >= 0){
            addressModel.has_change = "1";
            $scope.userAddressList[addressIndex] = angular.copy(addressModel);
        } else {
            addressModel.has_change = "0";
            $scope.userAddressList.push(addressModel);
        }
        
        if($scope.pageName === "delivery"){
            updateAddress($scope.userAddressList);
        }
        else{
            $rootScope.$broadcast('loading:hide');
            if($rootScope.profile){
                $rootScope.profile.address = $scope.userAddressList;
            }
            $scope.goBack();
        }
    };

    $scope.deleteAddress = function () {
        if (addressIndex >= 0) {
            var buttons = [
                    {text: 'Cancel'},
                    {
                        text: '<b class="b1_body_text_red">Delete</b>',
                        type: 'button button-stable',
                        onTap: function () {
                            if($scope.address.id > 0){
                                $scope.address.is_delete = "1";
                            }
                            // parse data model view -> data model
                            var addressModel = UserService.parseAddressModel($scope.address);
                            $scope.userAddressList[addressIndex] = angular.copy(addressModel);
                            updateAddress($scope.userAddressList);
                        }
                    }
                ];
            Utilities.showPopup($scope, '', constant.language.msg_delete_address, buttons);
        }
    };

    $scope.btnBackClicked = function () {
        $scope.goBack();
    };

    // PRIVATE FUNCTION
    function initNewAddress() {
        var dfd = $q.defer();
        UserService.parseItemAddressData(addressDefault).then(function (addressResult) {
                if(addressResult){
                    $rootScope.$broadcast('loading:hide');
                    dfd.resolve(addressResult);
                }
            }, function(error){
                $rootScope.$broadcast('loading:hide');
                console.log('parseAddressData: ' + error);
                 dfd.reject(error);
        });
        return dfd.promise;
    };
    function _initAddress(){
        var dfd = $q.defer();
        GlobalUser.getUser().then(function (user) {
                if (!user) {
                    return;
                }
                userData = angular.copy(user);
                $scope.userAddressList = user.address;
                var address = addressDefault;
                if(addressIndex){
                    address = user.address[addressIndex];
                }
                UserService.parseItemAddressData(address).then(function (addressResult) {
                        if(addressResult){
                            //$scope.address = $scope.userAddressList = angular.copy(addressResult);
                            dfd.resolve(addressResult);
                            $rootScope.$broadcast('loading:hide');
                        }
                    }, function(error){
                        dfd.reject(error);
                        console.log('parseAddressData: ' + error);
                });
            });
        return dfd.promise;
    }
    var updateAddress = function(addressList){
        userData.address = addressList;       
        $rootScope.$broadcast('loading:show');
        UserService.updateProfile(constant.chain_id, userData).then(function(user){            
            var resultUser = UserService.parseLoginUserData(user);
            // TODO: update to local db
            GlobalUser.init(resultUser);
            $rootScope.$broadcast('loading:hide');
            $scope.goBack();            
        }, function(error){
            Utilities.showAlertError(constant.language.mess_title_error, error.message);
            $rootScope.$broadcast('loading:hide');
        });
    };
    
    $scope.$on("$ionicView.beforeLeave", function () {
        $rootScope.$broadcast('loading:show');
    });

    $scope.$on("$ionicView.enter", function () {
        $rootScope.$broadcast('loading:hide');
    });
});