skipQueueApp.controller('RestaurantShowCtrl', function ($rootScope, $scope, $cordovaGeolocation, GlobalRestaurant, RestaurantService, restaurant) {
    $scope.showMap = false;
    $scope.restaurant = {};

    $scope.init = function () {
        // Parse model data to view model
        var parseResult = $scope.parseData(RestaurantService.parseDataResInfo, restaurant);

        if (!$scope.isParseDataSuccess(parseResult)) {
            return;
        }
        $scope.restaurant = parseResult.obj;
        
        $scope.loadMap();
    };
    
    $scope.toggleMap = function () {
        $scope.showMap = !$scope.showMap;
    };

    $scope.loadMap = function () {
        //Load Google Map by Latitude and Longitude
        if (navigator.geolocation && google !== "undefined") {            
            
            var coords = new google.maps.LatLng($scope.restaurant.latitude, $scope.restaurant.longitude);
            var myOptions = {
                zoom: 15,
                center: coords,
                mapTypeControl: false,
                navigationControlOptions: {
                    style: google.maps.NavigationControlStyle.SMALL
                },
                mapTypeId: google.maps.MapTypeId.ROADMAP
            };
            var map = new google.maps.Map(document.getElementById('map-container'), myOptions);
            
            var marker = new google.maps.Marker({
                position: coords,
                map: map,
                title: "Your place is here!"
            });       
            
            var posOptions = {
                enableHighAccuracy: true,
                timeout: 20000,
                maximumAge: 0
            };
            
            $cordovaGeolocation.getCurrentPosition(posOptions).then(function (pos) {
                var curLatLng = new google.maps.LatLng(pos.coords.latitude, pos.coords.longitude);
                var currentLoc = new google.maps.Marker({
                    position: curLatLng,
                    map: map,
                    title: "Current location"
                });
                
                // find way from current location to restaurant location
                var directionsService = new google.maps.DirectionsService();
                var directionsDisplay = new google.maps.DirectionsRenderer({
                    suppressMarkers: true
                });  
                directionsDisplay.setMap(map);
                
                directionsService.route({
                    origin: curLatLng, 
                    destination: coords, 
                    // Note that Javascript allows us to access the constant
                    // using square brackets and a string value as its
                    // "property."
                    travelMode: google.maps.TravelMode.DRIVING
                }, function(response, status) {
                    if (status === google.maps.DirectionsStatus.OK) {
                      directionsDisplay.setDirections(response);
                    } else {
                      console.log('Directions request failed due to ' + status);
                    }
                });                
                
            }, function(err) {
                console.log(err);
            });
            
//            google.maps.event.addListener(marker, 'click', function () {
//                
//            });
        }
        else {
            console.log('Cannot call Google Map');
        }
        //End Google Map script 
    };

    $scope.goToRestaurantMenuPage = function (resId) {
        // Update resId in globalRestaurant with the choosen resId
        var globalRes = GlobalRestaurant.getAll();
        globalRes.resId = resId;

        GlobalRestaurant.setResData(globalRes);
        // Go to restaurant menu
        $scope.goToPage(page_name.res_menu);
    };
    
    $scope.$on("$ionicView.beforeLeave", function(){
        $rootScope.$broadcast('loading:show');
    });

    $scope.$on("$ionicView.enter", function(){
        $rootScope.$broadcast('loading:hide');
    });
    
    $scope.init();
});