skipQueueApp.service('OrderCart', function (uuid2, $rootScope) {
    var _cart = {};

    var init = this.init = function (cart) {
        if (cart) {
            _cart = cart;
        } else {
            _cart = {
                dishes: {},
                resId: null,
                resName: null,
                userId: null,
                currency: null,
                selectedOrderType: constant.all_order_type,
                dateTime: null,
                address: null,
                grandTotal: null,
                phone: null,
                orderId: null,
                orderNumber:  null,
                promotion: null,
                freeAddon: null,
                tempItem: null,
                // HocOT - add 2 attributes to keep track of the last sub-cat and sub-sub-cat 
                currentSubCat: null,
                currentSubSubCat: null
            };
        }
    };

    var getItemCount = this.getItemCount = function () {
        var total = 0;
        angular.forEach(Object.keys(_cart.dishes), function(guid){
            total += _cart.dishes[guid].quantity;
        });
        
        return total;
    };

    this.getCart = function () {
        return _cart;
    };
    
    this.getDish = function(id){
        return _cart.dishes[id];
    };
    
    this.getRecommendation = function(id){
        var recommendation;
        angular.forEach(Object.keys(_cart.dishes), function(guid){
            var dish = _cart.dishes[guid];
            if(dish.isRecommendation && dish.parentId === id){
                recommendation = dish;
            }
        });
        return recommendation;
    };
    
    this.getCusOfDish = function(dish, cusId){
        if(!dish) return null;
        var cus;
        var cuses = dish.customizations;
        for (var i = 0; i < cuses.length; i++) {
            if (cuses[i].id===cusId) {
                cus = cuses[i];
            }
        }
        return cus;
    };
    
    this.getOptionOfCus = function(cus, optId){
        if(!cus) return null;
        var opt;
        var opts = cus.options;
        for (var i = 0; i < opts.length; i++) {
            if (opts[i].id===optId) {
                opt = opts[i];
            }
        }
        return opt;
    };
    
    this.addItemToCart = function (item) {
        var guid = generateGUID();
        item.guid = guid;
        _cart.dishes[guid] = item;
        _cart.currency = item.currency;
        getItemCount();
        return guid;
    };
    
    this.updateItemInCart = function(item, guid){
        item.guid = guid;
        _cart.dishes[guid] = item;
        getItemCount();
    };
    
    this.removeFromCart = function(guid){
        var removedItem = null;
        var recommendation = this.getRecommendation(guid);
        if(recommendation){
            removedItem = _cart.dishes[recommendation.guid];;
            delete _cart.dishes[recommendation.guid];
        } else {
            removedItem = _cart.dishes[guid];
            delete _cart.dishes[guid];
        }
        getItemCount();
        $rootScope.$broadcast('changeItemInCart', removedItem);
    };

    this.setResId = function (resId) {
        _cart.resId = resId;
    };
    
    this.setGrandTotal = function(value){
        _cart.grandTotal = value;
    };
    this.setSelectedOrderType = function (orderTypeId) {
        _cart.selectedOrderType = orderTypeId;
    };
    this.setDateTime = function (dateTime) {
        _cart.dateTime = dateTime;
    };
    this.setAddress = function (address) {
        _cart.address = address;
    };
    this.setPhone = function (phone) {
        _cart.phone = phone;
    };
    this.setResName = function(name){
        _cart.resName = name;
    };
    
    this.setOrderId = function(orderId){
        _cart.orderId = orderId;
    };
    
    this.setCurrency = function(currency){
        _cart.currency = currency;
    };
    
    this.setOrderNumber = function(orderNumber){
        _cart.orderNumber = orderNumber;
    };
    
    this.setTempItem = function(item){
        _cart.tempItem = item;
    };
    
    this.clearAll = function(){
        init();
    };
    
    this.clearExceptOrderIdAndOrderNumber = function(){
        var id = _cart.orderId;
        var number = _cart.orderNumber;
        init();
        _cart.orderId = id;
        _cart.orderNumber = number;
    };
    
    this.setPromotion = function(promo){
        _cart.promotion = promo;
    };
    
    this.clearPromotion = function(){
        _cart.promotion = null;
        _cart.freeAddon = null;
        angular.forEach(Object.keys(_cart.dishes), function(guid){
            var dish = _cart.dishes[guid];
            dish.price = dish.selectedSku.price;
            dish.newTotalPrice = dish.totalPrice;
        });
    };
    
    // Input: name of free addon dish. Server send this string to client and client just show it.
    this.setFreeAddon = function(freeAddon){
        _cart.freeAddon = freeAddon;
    };
    
    // Input: list of applied dish
    var updateDishWithPromotion = this.updateDishWithPromotion = function(guid, newTotalPrice){
        _cart.dishes[guid].newTotalPrice = parseFloat(newTotalPrice).toFixed(constant.number_after_comma);
    };
    
    this.getItemsByDishId = function(dishId){
        var items = $filter('filter')(_cart.dishes, {id: dishid});
        return items;
    };
    
    var generateGUID = this.generateGUID = function(){
        return uuid2.newguid();
    };
    
    this.clearTempItem = function(){
        _cart.tempItem = null;
    };
    
    this.setSubCat = function(subCat) {
        _cart.currentSubCat = subCat;
    };
    
    this.setSubSubCat = function(subSubCat) {
        _cart.currentSubSubCat = subSubCat;
    };
    
    this.clearLastSelection = function() {
        _cart.currentSubCat = null;
        _cart.currentSubSubCat = null;
    };
});