skipQueueApp.service('OrderService', function (ConnectionService, GlobalUser, Order, OrderCart, $q, Utilities, PromotionCalculator, GlobalStaff) {
    this.getCurrentOrderType = function () {
        return OrderCart.getCart()['selectedOrderType'];
    };

    this.getOrderList = function (chainKey, userId, localization) {
        var dfd = $q.defer();
        var rawParams = [
            {
                prm_name: params_name.chain_id,
                prm_value: chainKey
            }, {
                prm_name: params_name.user_id,
                prm_value: userId
            }, {
                prm_name: params_name.language_code,
                prm_value: localization
            }
        ];

        ConnectionService.get(api_url.track_order_list, rawParams).then(function (data) {
            dfd.resolve(data);
        }, function (error) {
            dfd.reject(error);
        });

        return dfd.promise;
    };
    
    this.getOrderDetail = function(userId, orderId, orderTypeId){
        var dfd = $q.defer();
        
        var rawParams = [{
            prm_name: params_name.chain_id,
            prm_value: constant.chain_id
        }, {
            prm_name: params_name.user_id,
            prm_value: userId
        }, {
            prm_name: params_name.order_id,
            prm_value: orderId
        }, {
            prm_name: params_name.order_type_id,
            prm_value: orderTypeId
        }];
    
        ConnectionService.get(api_url.get_order_info, rawParams).then(function (data) {
            dfd.resolve(data);
        }, function (error) {
            dfd.reject(error);
        });
        
        return dfd.promise;
    };

    this.addToCart = function (dish) {
        var item = parseData(dish);
        //if dish is a recommend
        if (dish.isRecommendation) {
            //check this recommend is exist in cart?
            var oldRecommend = OrderCart.getRecommendation(dish.parentId);
            //if exist: update
            if (oldRecommend && oldRecommend.id === dish.id) {
                OrderCart.updateItemInCart(dish, dish.parentId);
                return oldRecommend.guid;
            } else if (oldRecommend) {
                //delete oldRecommend
                OrderCart.removeFromCart(oldRecommend.guid);
            }
        }

        //add dish to cart
        var guid = OrderCart.addItemToCart(item);
        applyPromoToItemWhenAddToCart(item);
        return guid;
    };

    this.updateItemInCart = function (dish, guid) {
        var item = parseData(dish);
        var orgItem = OrderCart.getDish(guid);
        if (orgItem) {
            OrderCart.updateItemInCart(item, guid);
            applyPromoToItemWhenAddToCart(item);
            return guid;
        }
    };

    var parseData = this.parseData = function (dish) {
        var result = {
            id: dish.id,
            name: dish.name,
            hasPromotion: dish.hasPromotion,
            quantity: dish.quantity,
            selectedSku: {
                id: dish.selectedSku.skuId,
                name: dish.selectedSku.name,
                basePrice: dish.selectedSku.basePrice,
                price: dish.selectedSku.price,
                hasRecommendation: dish.selectedSku.hasRecommendation
            },
            specialIntrustion: dish.specialIntrustion,
            price: dish.selectedSku.price,
            orgPrice: dish.selectedSku.orgPrice,
            basePrice: dish.selectedSku.basePrice,
            currency: dish.currency,
            category: dish.category,
            subCatId: dish.subCatId,
            customizations: [],
            totalPrice: dish.totalPrice.toFixed(constant.number_after_comma),
            newTotalPrice: dish.totalPrice.toFixed(constant.number_after_comma),
            isRecommendation: dish.isRecommendation,
            cusId: dish.cusId,
            parentId: dish.parentId
        };

        angular.forEach(dish.customizations, function (cus) {
            var temp = {
                id: cus.id,
                name: cus.name,
                paidAfterValue: cus.paidAfterValue,
                paidAfterPrice: cus.paidAfterPrice ? cus.paidAfterPrice.toFixed(constant.number_after_comma) : 0, //cus.paidAfterPrice, //cus.paidAfter,
                paidAfter: cus.paidAfter,
                options: []
            };

            angular.forEach(cus.options, function (opt) {
                if (opt.quantity > 0) {
                    temp.options.push(opt);
                }
            });

            if (temp.options.length > 0) {
                result.customizations.push(temp);
            }
        });

        return result;
    };
    
    this.parseDataTrackOrderList = function(data){
        var result = {
            currentOrder: [],
            pastOrder: []
        };
        
        if(data.current.length){
            angular.forEach(data.current, function(order){
                var temp = new Order();
                temp.parseData(order);
                result.currentOrder.push(temp);
            });
        }
        
        if(data.past.length){
            angular.forEach(data.past, function(order){
                var temp = new Order();
                temp.parseData(order);
                result.pastOrder.push(temp);
            });
        }
        
        return result;
    };
    
    this.parseDataOrderDetail = function(order){
        var result = new Order();
        result.parseData(order);
        result.bill.parseData(order);
        return result;
    };

    // Send confirm order to server. Ongoing.
    this.validateOrder = function () {
        var cart = OrderCart.getCart();
        var orderType = cart.selectedOrderType;
        switch (orderType) {
            case constant.order_type_child_pickup_now:
            case constant.order_type_child_pickup_later:
            {
                return validatePickUpOrder(cart, api_url.validate_order_pickup);
                break;
            }
            case constant.order_type_child_delivery_now:
            case constant.order_type_child_delivery_later:
            {
                return validateDeliveryOrder(cart, api_url.validate_delivery_order);
                break;
            }
            default:
                console.log('default error');
                break;
        }
    };

    this.confirmOrder = function () {
        var cart = OrderCart.getCart();
        var orderType = cart.selectedOrderType;
        switch (orderType) {
            case constant.order_type_child_pickup_now:
            case constant.order_type_child_pickup_later:
            {
                return confirmPickUpOrder(cart, api_url.make_payment_pick_up_order);
                break;
            }
            case constant.order_type_child_delivery_now:
            case constant.order_type_child_delivery_later:
            {
                return confirmDeliveryOrder(cart, api_url.make_payment_delivery_order);
                break;
            }
            default:
                console.log('default error');
                break;
        }
    };

    var parseDishToConfirmOrder = this.parseDishToConfirmOrder = function (dishes) {
        var result = [];
        angular.forEach(dishes, function (dish) {
//            console.log("ABC");
//            console.log(dish);
            var temp = {};
            temp[params_name.dish_id] = dish.id;
            temp[params_name.sku_id] = dish.selectedSku.id;
            temp[params_name.dish_total_price] = dish.totalPrice,//.toFixed(constant.number_after_comma);
            temp[params_name.quantity] = dish.quantity;
            temp[params_name.customizations] = [];
            temp[params_name.dish_special_instruction] = dish.specialIntrustion;
            temp[params_name.is_recommendations] = dish.isRecommendation ? 1: 0;
            temp[params_name.cus_id] = dish.cusId;

            angular.forEach(dish.customizations, function (cus) {
                angular.forEach(cus.options, function (opt) {
                    var optTemp = {};
                    optTemp[params_name.cus_opt_id] = opt.id;
                    optTemp[params_name.cus_opt_quantity] = opt.quantity;
                    temp[params_name.customizations].push(optTemp);
                });
            });

            result.push(temp);
        });

        return result;
    };

    // Parse data
    this.parseConfirmDetail = function (data) {
        var result = {
            id: data.order_id,
            no: data.order_no,
            subTotal: data.sub_total,
            grandTotal: data.grand_total,
            priceIncludeGST: data.price_include_gst,
            dishes: [],
            discountChargeTax: []
        };

        angular.forEach(data.dishes, function (dish) {
            var temp = {
                id: dish.dish_id,
                name: dish.dish_name,
                quantity: Utilities.parseNumber(dish.quantity),
                amount: dish.amount,
                sku_name: dish.sku_name,
                sku_price: dish.sku_price,
                specialInstruction: dish.special_instruction,
                customizations: []
            };

            angular.forEach(dish.customizations, function (opt) {
                var optTemp = {
                    id: opt.cust_opt_id,
                    name: opt.cust_opt_name,
                    quantity: Utilities.parseNumber(opt.cust_opt_quantity),
                    price: opt.cust_opt_price,
                    amount: opt.cust_opt_amount,//.toFixed(constant.number_after_comma)
                };
                
                // add extra to display paid after
                if (opt.extra !== undefined){
                    optTemp.extra = opt.extra;
                }

                temp.customizations.push(optTemp);
            });

            result.dishes.push(temp);
        });

        angular.forEach(data.discount_charge_tax, function (DCTObj) {
            var dtc = {
                name: DCTObj.name,
                amount: DCTObj.amount,//.toFixed(constant.number_after_comma)
            };

            result.discountChargeTax.push(dtc);
        });

        return result;
    };

    // private function
    var validatePickUpOrder = function (cart, url) {
        var dfd = $q.defer();
        
        GlobalStaff.getStaff().then(function (staff) {                        
            GlobalUser.getUser().then(function (user) {
                
                var rawParams = preparePickUpParams(cart, user, staff);

                // Call API to get categories by orderType, restaurantId, chainId, localization
                ConnectionService.post(url, rawParams).then(function (data) {
                    dfd.resolve(data);
                }, function (error) {
                    dfd.reject(error);
                });
            });
        });                

        return dfd.promise;
    };

    var validateDeliveryOrder = function (cart, url) {
        var dfd = $q.defer();
        
        GlobalStaff.getStaff().then(function (staff) {                        
            GlobalUser.getUser().then(function (user) {
                
                var rawParams = prepareDeliveryParams(cart, user, staff);

                // Call API to get categories by orderType, restaurantId, chainId, localization
                ConnectionService.post(url, rawParams).then(function (data) {
                    dfd.resolve(data);
                }, function (error) {
                    dfd.reject(error);
                });
            });
        });                

        return dfd.promise;
    };

    var confirmPickUpOrder = function (cart, url) {
        var dfd = $q.defer();
        
        GlobalStaff.getStaff().then(function (staff) {                        
            GlobalUser.getUser().then(function (user) {
                
                var rawParams = preparePickUpParams(cart, user, staff);

                // Call API to get categories by orderType, restaurantId, chainId, localization
                ConnectionService.post(url, rawParams).then(function (data) {
                    dfd.resolve(data);
                }, function (error) {
                    dfd.reject(error);
                });
            });
        });                

        return dfd.promise;
    };

    var confirmDeliveryOrder = function (cart, url) {
        var dfd = $q.defer();
        
        GlobalStaff.getStaff().then(function (staff) {                        
            GlobalUser.getUser().then(function (user) {
                
                var rawParams = prepareDeliveryParams(cart, user, staff);

                // Call API to get categories by orderType, restaurantId, chainId, localization
                ConnectionService.post(url, rawParams).then(function (data) {
                    dfd.resolve(data);
                }, function (error) {
                    dfd.reject(error);
                });
            });
        });
                
        return dfd.promise;
    };

    var prepareSameConfirmRawParams = this.prepareSameConfirmRawParams = function (cart, user, staff) {                        
        var deviceInfo = Utilities.getDeviceInfo();
        var rawParams = [
            {
                prm_name: params_name.chain_id,
                prm_value: constant.chain_id
            },
            {
                prm_name: params_name.order_type,
                prm_value: cart.selectedOrderType
            }, {
                prm_name: params_name.restaurant_id,
                prm_value: cart.resId
            }, {
                prm_name: params_name.user_id,
                prm_value: user.user_id
            }, {
                prm_name: params_name.order_dishes,
                prm_value: parseDishToConfirmOrder(cart.dishes)
            }, {
                prm_name: params_name.is_skipque,
                prm_value: constant.is_skipque
            }
        ];

        if (deviceInfo) {
            rawParams = rawParams.concat([
                {
                    prm_name: params_name.os_type,
                    prm_value: deviceInfo.platform
                }, 
                {
                    prm_name: params_name.os_version,
                    prm_value: deviceInfo.version
                }, 
                {
                    prm_name: params_name.device_name,
                    prm_value: deviceInfo.model
                }, 
                {
                    prm_name: params_name.app_version,
                    prm_value: constant.app_version
                }
            ]);
        }

        if (cart.orderId) {
            rawParams.push({
                prm_name: params_name.order_id,
                prm_value: cart.orderId
            });
        }
        
        if (cart.promotion) {
            rawParams.push({
                prm_name: params_name.promotion_code,
                prm_value: cart.promotion.code
            });
        }
        
        if (staff) {
            rawParams.push({
                prm_name: params_name.staff_id,
                prm_value: staff.staff_id
            });
        }

        return rawParams;
    };

    var preparePickUpParams = function (cart, user, staff) {
        var rawParams = prepareSameConfirmRawParams(cart, user, staff);
        rawParams = rawParams.concat([
            {
                prm_name: params_name.user_phone,
                prm_value: cart.phone
            }
        ]);

        if (cart.selectedOrderType == constant.order_type_child_pickup_later) {
            rawParams.push({
                prm_name: params_name.order_time,
                prm_value: cart.dateTime
            });
        }

        return rawParams;
    };

    var prepareDeliveryParams = function (cart, user, staff) {
        var rawParams = prepareSameConfirmRawParams(cart, user, staff);
        rawParams = rawParams.concat([
            {
                prm_name: params_name.user_phone,
                prm_value: cart.phone
            }, {
                prm_name: params_name.address_id,
                prm_value: cart.address.id
            }
        ]);

        if (cart.selectedOrderType == constant.order_type_child_delivery_later) {
            rawParams.push({
                prm_name: params_name.order_time,
                prm_value: cart.dateTime
            });
        }

        return rawParams;
    };

    // Input: promotion, order cart
    this.applyPromotionToCart = function (promo, cart) {
        
        if (promo.discountType == constant.free_item) {         
            var result = PromotionCalculator.calculatePromotion(promo);
            applyPromotionWithAddonItems(result);
        } else {
            applyPromotionWithoutAddonItems(promo, cart.dishes);
        }
    };

    var applyPromoToItemWhenAddToCart = this.applyPromoToItemWhenAddToCart = function (item) {
        var cart = OrderCart.getCart();
        if (cart.promotion) {
            if (cart.promotion.discountType == constant.discount_by_percent ||
                    cart.promotion.discountType == constant.discount_by_value) {
                var items = [];
                items.push(item);
                applyPromotionWithoutAddonItems(cart.promotion, items);
            }
        }
    };

    var applyPromotionWithAddonItems = function (res) {
        var addonDishes = [];
        angular.forEach(res.promotionDishes, function (addonDish) {
            var temp = {
                name: addonDish.name
            };

            addonDishes.push(temp);
        });

        OrderCart.setFreeAddon(addonDishes);
    };

    var applyPromotionWithoutAddonItems = function (promo, items) {
        angular.forEach(items, function (item) {
            var newTotalPrice = PromotionCalculator.calculatePromotion(promo, item);
            OrderCart.updateDishWithPromotion(item.guid, newTotalPrice);
        });
    };
    
    //Feedback
    //parse order list on feedback
    this.parseOrderList = function (data) {
        var list=[];
        angular.forEach(data, function(item){
            list.push({order_id: item.id,
                    order_display_name: item.order_display_name});
        });
        return list;
    };
    
    this.parseItemOrder = function (data) {
        return {order_id: data.order_id,
                res_id:data.res_id,
                item_list: _parseItemsDishOfFeedback(data.item_list),
                service_list: _parseItemsServiceOfFeedback(data.service_list),
                feedback_comment:''};
    };
    this.parseItemFromOrderCart = function (data) {
        return {order_id: data.order_id,
                order_display_name: "",
                res_id:data.res_id,
                item_list: _parseItemsDishOfFeedbackFromOrderCard(data.item_list),
                service_list: _parseItemsServiceOfFeedback(data.service_list),
                feedback_comment:''};
    };
    function _parseItemsServiceOfFeedback(service_list){
        var list=[];
        angular.forEach(service_list, function(item){
            list.push({service_id: item.id, 
                        service_name: item.name, 
                        rating : 0,
                        isSelectedStar_1: true,
                        isSelectedStar_2: true,
                        isSelectedStar_3: true,
                        isSelectedStar_4: true,
                        isSelectedStar_5: true
                   });
        });
        return list;
    }
    function _parseItemsDishOfFeedbackFromOrderCard(item_list){
        var list=[];
//        var fav_count = 0;
//        if (item_list.dish_analytics.length > 0) {
//            fav_count = item_list.dish_analytics[0].total_skipque_fav_count;
//        }
        angular.forEach(item_list, function(item){
            list.push({id: item.dish_id, 
                        name: item.dish_name, 
                        rating : 0, 
                        isLiked: item.is_favorite,
                        favCounter: item.favorite_count,
                        isSelectedStar_1: true,
                        isSelectedStar_2: true,
                        isSelectedStar_3: true,
                        isSelectedStar_4: true,
                        isSelectedStar_5: true
                    });      
        });
        return list;
    }
    
    function _parseItemsDishOfFeedback(item_list){
        var list=[];
        angular.forEach(item_list, function(item){
            list.push({id: item.dish_id, 
                        name: item.dish_name, 
                        rating : 0, 
                        isLiked: item.is_favorite,
                        favCounter: item.favorite_count,
                        isSelectedStar_1: true,
                        isSelectedStar_2: true,
                        isSelectedStar_3: true,
                        isSelectedStar_4: true,
                        isSelectedStar_5: true
                    });      
        });
        return list;
    }
    
    this.getListOrderFeedback = function(chainKey, userID, localization){
        var dfd = $q.defer();
        var rawParams = [{
                prm_name: params_name.chain_id,
                prm_value: chainKey
            },
            {
                prm_name: params_name.user_id,
                prm_value: userID
            },
            {
                prm_name: params_name.language_code,
                prm_value: localization
            }
        ];
        // Call API to get_list_order_feedback by userId, chainId, localization
        ConnectionService.get(api_url.get_list_order_feedback, rawParams).then(function (data) {
                dfd.resolve(data);
            }, function (error) {
                dfd.reject(error);
            });
            return dfd.promise;
    };
    
    this.getListItemFeedback = function(chainKey, userID, orderId, localization){
        var dfd = $q.defer();
        var rawParams = [{
                prm_name: params_name.chain_id,
                prm_value: chainKey
            },
            {
                prm_name: params_name.user_id,
                prm_value: userID
            },
            {
                prm_name: params_name.order_id,
                prm_value: orderId
            },
            {
                prm_name: params_name.language_code,
                prm_value: localization
            }
        ];
        // Call API to get_list_order_feedback by userId, chainId, localization
        ConnectionService.get(api_url.get_list_item_feedback, rawParams).then(function (data) {
                dfd.resolve(data);
            }, function (error) {
                dfd.reject(error);
            });
            return dfd.promise;
    };
    
    this.confirmFeedback = function(chainKey, userID, order){
        var dfd = $q.defer();
        var rawParams = [{
                prm_name: params_name.chain_id,
                prm_value: chainKey
            },
            {
                prm_name: params_name.user_id,
                prm_value: userID
            },
            {
                prm_name: params_name.order_id,
                prm_value: order.order_id
            },
            {
                prm_name: params_name.service_list,
                prm_value: order.service_list
            },
            {
                prm_name: params_name.item_list,
                prm_value: order.item_list
            },
            {
                prm_name: params_name.feedback_comment,
                prm_value: order.feedback_comment
            }
        ];
        // Call API to get_list_order_feedback by userId, chainId, localization
        ConnectionService.post(api_url.confirm_feedback, rawParams).then(function (data) {
                dfd.resolve(data);
            }, function (error) {
                dfd.reject(error);
            });
        return dfd.promise;
    };
    
    this.validateConfirmFeedback = function (itemOrder){
        var result = false;
        if(!itemOrder || (itemOrder && !itemOrder.order_id)){
            Utilities.showAlertError(constant.language.mess_title_error, constant.language.msg_fb_choose_order);
            return false;
        }
        if(itemOrder.item_list && itemOrder.item_list.length > 0){
            angular.forEach(itemOrder.item_list, function(item){
                if(item.rating > 0)
                    result = true;
            });
        }
        if(itemOrder.feedback_comment)
            result = true;
        
        if(!result){
            Utilities.showAlertError(constant.language.mess_title_error, constant.language.msg_fb_choose_option_order);
        }
        return result;
    };
    
    this.clearOrderCartAndSetTempItem = function(guid){
        var tempItem = OrderCart.getDish(guid);
        OrderCart.removeFromCart(guid);
        OrderCart.setTempItem(tempItem);
    };
});