skipQueueApp.service('PromotionCalculator', function($q, $filter, Utilities, RestaurantService){
    // Caculate price for dish in cart with promo
    // Input: promotion, dish
    // Output: Dish with newPrice or free item list
    
    this.calculatePromotion = function(promo, dish){
        if(promo.discountType == constant.free_item){
            return applyPromoWithFreeItem(promo);
        } else {
            if(promo.applyAllCategory){
                return applyPromoWithAllCat(promo, dish);
            } else {
                return applyPromoWithSpecificCat(promo, dish);
            }
        }
    };
    
    // Private function
    // Input: promotion with free item
    // just add free item to order cart
    // Output: free item in array type
    var applyPromoWithFreeItem = function(promo){
        var promotionDishes = [];
        promotionDishes.push(promo.freeItem);
        return {
            promotionDishes: promotionDishes
        };
    };
    
    
    // Input: promotion, item in orderCart
    var applyPromoWithAllCat = function(promo, item){
        return applyPromoForItem(promo, item);
    };
    
    var applyPromoWithSpecificCat = function(promo, item){
        if(isCanApplyPromo(promo, item)){
            return applyPromoForItem(promo, item);
        } else {
            return item.totalPrice;
        }
    };
    
    var applyPromoForItem = function(promo, item){
        var skuPrice = parseFloat(item.price); //parseFloat(item.basePrice);
        if(promo.discountType == constant.discount_by_percent){
            var discount = (Utilities.parseToFloat(promo.discountValue) / 100 ) * skuPrice;
            skuPrice -= discount;
        } else if (promo.discountType == constant.discount_by_value){
            skuPrice -= Utilities.parseToFloat(promo.discountValue);
        }
        
        if(skuPrice < 0){
            skuPrice = 0;
        }
        
        item.selectedSku.price = skuPrice;
        var result = RestaurantService.getDishTotalPrice(item);
        
        // exchange data between item.selectedSku.price and item.price
        item.selectedSku.price = item.price;
        item.price = skuPrice;
        return result;
    };
    
    // Input: promotion, item to apply promo
    // Output: new total price of item after apply promo
    // Work flow:
    // Check if item is in apply dishes, get apply info(info)
    // if true, check if info only apply for sku
    // if item has apply sku, return true
    // if item dosen't has apply sku, return false
    // if info dosen't have apply for sku (mean apply for all sku) return true
    // if items isn't in apply dishes, check if item is in apply category
    // get apply category by item.category.id (info)
    // if info has value, check if info has subCatId (mean only apply for this sub category)
    // if info.cusbCatId == item.subCatId return true
    // if info.cusbCatId != item.subCatId return false
    // if info dosen't have subCatId (mean apply for all item in category) return true
    // if info is null or empty, return false
    var isCanApplyPromo = function(promo, item){
        if(isInApplyDishes(promo.applyDishes, item)){
            return true;
        };
        
        if(isInApplyCategory(promo.applyCategories, item)){
            return true;
        }
        return false;
    };
    
    var isInApplyDishes = function(list, item){
        var temp = $filter('filter')(list, {id: item.id});
        if(temp.length > 0){
            if(temp.skuId){
                return temp.skuId == item.selectedSku.id;
            } else {
                return true;
            }
        }
        
        return false;
    };
    
    var isInApplyCategory = function(cats, item){
        var temp = $filter('filter')(cats, {id: item.category.id});
        if(temp.length > 0){
            if(temp.subCatId){
                return temp.subCatId == item.subCatId;
            } else {
                return true;
            };
        }
        
        return false;
    };
});