skipQueueApp.service('PromotionSrv', function ($rootScope, $q, ConnectionService, GlobalUser, OrderService, OrderCart, Utilities, Promotion) {

    var _promotionList = null;

    this.getPromotionList = function (chainId, resId, userId, orderTypeId, datetime, localization) {
        var rawParams = [{
                prm_name: params_name.chain_id,
                prm_value: chainId
            }, {
                prm_name: params_name.restaurant_id,
                prm_value: resId
            }, {
                prm_name: params_name.user_id,
                prm_value: userId
            }, {
                prm_name: params_name.language_code,
                prm_value: localization
            }];

        if (orderTypeId) {
            rawParams.push({
                prm_name: params_name.order_type_id,
                prm_value: orderTypeId
            });
        }

        if (datetime) {
            rawParams.push({
                prm_name: params_name.local_time,
                prm_value: datetime
            });
        }

        var dfd = $q.defer();

        // Call API to get promotion list
        ConnectionService.post(api_url.get_promotion_list, rawParams).then(function (data) {
            dfd.resolve(data);
        }, function (error) {
            $rootScope.$broadcast('loading:hide');
            dfd.reject(error);
        });

        return dfd.promise;
    };

    this.getPromotionDetail = function (promoId, localization) {
        var rawParams = [{
                prm_name: params_name.promotion_id,
                prm_value: promoId
            }, {
                prm_name: params_name.language_code,
                prm_value: localization
            }];


        var dfd = $q.defer();

        // Call API to get promotion list
        ConnectionService.get(api_url.get_promotion_detail, rawParams).then(function (data) {
            // if success
            // return data to controller
            dfd.resolve(data);
        }, function (error) {
            $rootScope.$broadcast('loading:hide');
            // if error
            // return error
            dfd.reject(error);
        });

        return dfd.promise;
    };

    /**
     * Input: List of promotion got from server
     * Ouput: List of promotion view model
     * */
    this.parseDataListPromo = function (resData) {
        var result = [];
        angular.forEach(resData, function (promo) {
            var temp = new Promotion();
            temp.parseData(promo);
            result.push(temp);
        });

        return result;
    };

    /**
     * Input: Promotion got from server
     * Ouput: Promotion view model
     * */
    this.parseDataPromoDetail = function (resData) {
        var result = new Promotion();
        result.parseData(resData);
        return result;
    };

    this.getPromoList = function () {
        return _promotionList;
    };

    this.setPromoList = function (list) {
        _promotionList = list;
    };

    // Business logic
    this.applyPromotion = function (promotion) {
        var dfd = $q.defer();
        validatePromotion(promotion).then(function (result) {
            var promo = new Promotion();
            promo.parseData(result);
            promo.setName(promo.name);
            promo.setDescription(promo.description);
            OrderCart.clearPromotion();
            OrderCart.setPromotion(promo);
            OrderService.applyPromotionToCart(promo, OrderCart.getCart());
            dfd.resolve();
        }, function (error) {
            dfd.reject(error);
        });
        return dfd.promise;
    };

    // Call API to validate promotion user pick
    var validatePromotion = this.validatePromotion = function (promotion) {
        var dfd = $q.defer();
        GlobalUser.getUser().then(function (user) {
            if (user) {
                var cart = OrderCart.getCart();
                var rawParams = prepareValidationPromoParams(cart, user, promotion.code);

                if (cart.selectedOrderType == constant.order_type_child_delivery_later ||
                        cart.selectedOrderType == constant.order_type_child_pickup_later) {
                    rawParams.push({
                        prm_name: params_name.local_time,
                        prm_value: cart.dateTime
                    });
                }

                // Call API validate promotion
                $rootScope.$broadcast('loading:show');
                ConnectionService.post(api_url.apply_promotion, rawParams).then(function (data) {
                    $rootScope.$broadcast('loading:hide');
                    dfd.resolve(data);
                }, function (error) {
                    $rootScope.$broadcast('loading:hide');
                    dfd.reject(error);
                });
            } else {
                //TODO: Log user not exist
                dfd.reject();
            }
        });

        return dfd.promise;
    };
    var prepareValidationPromoParams = this.prepareValidationPromoParams = function (cart, user, promoCode) {
        var rawParams = [
            {
                prm_name: params_name.order_type,
                prm_value: cart.selectedOrderType
            }, {
                prm_name: params_name.restaurant_id,
                prm_value: cart.resId
            }, {
                prm_name: params_name.user_id,
                prm_value: user.user_id
            }, {
                prm_name: params_name.promotion_code,
                prm_value: promoCode
            }
        ];

        return rawParams;
    };
});