/* global page_name, constant */

skipQueueApp.service('Utilities', function ($rootScope, $state, $ionicPopup, $filter, $cordovaDevice, $cordovaCamera,
        GlobalUser, $cordovaGeolocation, $q, OrderCart, $ionicLoading, $cordovaToast, ionicToast) {
    // prepare params for call API
    // Input: list of {prm_name, prm_value}
    // Output: Params send to api

    var _device = null;

    this.prepareParams = function (listOfParams) {
        // create empty object params = {}
        var params = {};
        // loop through listOfParams
        angular.forEach(listOfParams, function (param) {
            // set property and value for params from listOfParams
            params[param.prm_name] = param.prm_value;
        });

        // return params
        return params;
    };

    this.prepareUrl = function (baseUrl, listOfParams) {
        var url = baseUrl;
        angular.forEach(listOfParams, function (param) {
            // set property and value for params from listOfParams
            //params[param.prm_name] = param.prm_value;
            url += '/:' + param.prm_name;
        });

        return url;
    };

    this.parseData = function (callback, obj) {
        try {
            var data = callback(obj);
            return {
                result: true,
                obj: data
            };
        } catch (exc) {
            return {
                result: false,
                obj: {
                    errorCode: error_code.parse_data,
                    message: error_message.parse_data
                }
            };
        }
    };

    //value = 'aaa'->True, value='' or null or undefined -> false
    // caoth: reversed function name, should return true if empty or null !
    var emptyOrNull = this.emptyOrNull = function (value) {
        return !(angular.isUndefined(value) || value === null || value.toString().trim().length === 0);
    };
    
    var isEmpty = this.isEmpty = function (value) {
        return (angular.isUndefined(value) || value === null || value.toString().trim().length === 0);
    };

    var showAlert = this.showAlert = function (headerContent, messageContent, cb) {
        var alertPopup = $ionicPopup.alert({
            title: headerContent,
            template: messageContent
        });
        alertPopup.then(function (res) {
            if(cb){
                cb(res);
            }
        });
    };
    
    var jsHideToast = this.jsHideToast = function () {
        // https://github.com/rajeshwarpatlolla/ionic-toast
        ionicToast.hide();        
    }
    
    var jsShortToast = this.jsShortToast = function (txt) {
        // https://github.com/rajeshwarpatlolla/ionic-toast
        ionicToast.hide();
        ionicToast.show(txt, 'bottom', false, 2000);
    }
    
    var jsLongToast = this.jsLongToast = function (txt) {
        // https://github.com/rajeshwarpatlolla/ionic-toast
        ionicToast.hide();
        ionicToast.show(txt, 'bottom', false, 5000);
    }

    var showAlertError = this.showAlertError = function (headerContent, messageContent, cb) {
        /* caoth
        var alertPopup = $ionicPopup.alert({
            title: headerContent,
            template: messageContent,
            okType: 'button-assertive'
        });
        alertPopup.then(function (res) {
            if(cb){
                cb(res);
            }
        });
        */
       
        // caoth
        /*
        if (window.cordova) {
            $cordovaToast.showLongBottom(messageContent).then(function(success) {
                // success

            }, function (error) {
                // error

            });
        } else {
            console.log(messageContent);
        }
        */
       
        jsLongToast(messageContent);
       
    };
    
    var showShortAlertError = this.showShortAlertError = function (headerContent, messageContent, cb) {       
        /*
        if (window.cordova) {
            $cordovaToast.showShortBottom(messageContent).then(function(success) {
                // success

            }, function (error) {
                // error

            });
        } else {
            console.log(messageContent);
        }
        */
       
        jsShortToast(messageContent);
       
    };
    
    var showPopupAlertError = this.showPopupAlertError = function (headerContent, messageContent, cb) {
        var alertPopup = $ionicPopup.alert({
            title: headerContent,
            template: messageContent,
            okType: 'button-assertive'
        });
        alertPopup.then(function (res) {
            if(cb){
                cb(res);
            }
        });
    };

    // Input: title, message, callback function(true --> choose OK, false --> choose cancel)
    var showAlertErrorCallback = this.showAlertErrorCallback = function (headerContent, messageContent, callback) {
        var alertPopup = $ionicPopup.alert({
            title: headerContent,
            template: messageContent,
            okType:'button-assertive'
        });
        alertPopup.then(function (res) {
            if (res) {
                callback ? callback(true) : null;
            } else {
                callback ? callback(false) : null;
            }
        });
    };
    
    this.showConfirm = function (headerContent, messageContent, callback) {
        var confirmPopup = $ionicPopup.confirm({
            title: headerContent,
            template: messageContent
        });
        confirmPopup.then(function (res) {
            if (res) {
                callback ? callback(true) : null;
            } else {
                callback ? callback(false) : null;
            }
        });
    };

    this.showPopup = function ($scope, headerContent, messageContent, buttons) {
        // An elaborate, custom popup
        var popup = $ionicPopup.show({
            template: messageContent,
            title: headerContent,
            scope: $scope,
            buttons: buttons
        });
        return popup;
    };
    // Sort list by property, reverse
    this.sort = function (list, property, isReverse) {
        var result = null;
        result = $filter('orderBy')(list, property, isReverse);
        return result;
    };
    this.encodeImageUri = function (imageUri)
    {
        var c = document.createElement('canvas');
        var ctx = c.getContext("2d");
        var img = new Image();
        img.onload = function () {
            c.width = this.width;
            c.height = this.height;
            ctx.drawImage(img, 0, 0);
        };
        img.src = imageUri;
        var dataURL = c.toDataURL("image/png");
        return dataURL;
    };

    this.getCurrentPosition = function () {
//        $rootScope.$broadcast('loading:show');        
        var dfd = $q.defer();
        $ionicLoading.show({
//            template: constant.language.msg_getting_location // not run ?
            template: 'Getting your location...'
        });
        var posOptions = {timeout: 5000, enableHighAccuracy: false};
        $cordovaGeolocation.getCurrentPosition(posOptions).then(function (position) {
            $ionicLoading.hide();
            var lat = position.coords.latitude;
            var long = position.coords.longitude;
            dfd.resolve({
                lat: lat,
                long: long
            });
        }, function (error) {          
            $ionicLoading.hide();
            if (window.cordova) {
                $cordovaToast.showShortBottom(constant.language.err_msg.no_location).then(function(success) {
                    // success
                    dfd.resolve({
                        lat: null,
                        long: null
                    }); 
                }, function (error) {
                    // error
                    dfd.resolve({
                        lat: null,
                        long: null
                    });
                });
            } else {
                showAlert(constant.language.mess_title_error, constant.language.err_msg.no_location, function(result){
                    dfd.resolve({
                        lat: null,
                        long: null
                    }); 
                });
            }            
        });
        return dfd.promise;
    };
    
    this.getCurrentPositionWithoutErrorPopup = function () {
//        $rootScope.$broadcast('loading:show');
        var dfd = $q.defer();
        var posOptions = {timeout: 5000, enableHighAccuracy: false};
        $cordovaGeolocation.getCurrentPosition(posOptions).then(function (position) {
            var lat = position.coords.latitude;
            var long = position.coords.longitude;
            dfd.resolve({
                lat: lat,
                long: long
            });
        }, function (error) {
            dfd.resolve({
                lat: null,
                long: null
            });
        });
        return dfd.promise;
    };

    // format: 'yyyy-MM-dd HH:mm:ss Z'
    // ToDo: caoth: 1 function with 1 line ...
    this.formatDateTime = function (format, dateTime) {
        var result = $filter('date')(dateTime, format);
        return result;
    };

    this.isLikedDish = function (list) {
        var dfd = $q.defer();
        GlobalUser.getUser().then(function (user) {
            if (user) {
                var result = $filter('filter')(list, {customer_id: user.user_id});
                if (result && result.length > 0)
                    dfd.resolve(true);
                //return true;
                else
                    dfd.resolve(false);
                //return false;
            } else {
                dfd.resolve(false);
            }
        });
        return dfd.promise;
    };

    this.initDeviceInfo = function () {
        document.addEventListener("deviceready", function () {
            _device = $cordovaDevice.getDevice();
        }, false);
    };

    this.getDeviceInfo = function () {
//        if(_device == null) {
//            _device = $cordovaDevice.getDevice();
//        }
        
//        $cordovaToast.showShortBottom(_device.model).then(function(success) {
//            // success
//             
//        }, function (error) {
//            // error
//            
//        });
        
        return _device;
    };
    // arr=[{id:XX, name:XX},...]
    this.getNameOfArr = function (arr, key) {
        var name = '';
        arr.forEach(function (obj) {
            if (obj.id === key) {
                name = obj.name;
            }
        });
        return name;
    };
    this.getIdOfArr = function (arr, name) {
        var id = '';
        arr.forEach(function (obj) {
            if (obj.name === $.trim(name)) {
                id = obj.id;
            }
        });
        return id;
    };

    this.showLoading = function () {
        $ionicLoading.show({
            template: 'Loading...',
            number: 200000
        });
    };

    this.hideLoading = function () {
        $ionicLoading.hide();
    };

    this.getOrderTypeNameById = function (orderTypeId) {
        switch (orderTypeId) {
            case constant.order_type_child_pickup_now:
            case constant.order_type_child_pickup_later:
            {
                return constant.order_type_pickup;
            }
            case constant.order_type_child_delivery_later:
            case constant.order_type_child_delivery_now:
            {
                return constant.order_type_delivery;
            }
        }
        return '';
    };

    // Input string datetime
    // Output date result
    this.getDateFromDatetime = function (datetime) {
        var arr = datetime.split(/[- :]/);
        var temp = new Date(arr[0], arr[1] - 1, arr[2], arr[3], arr[4], arr[5]);
        var sTemp = temp.toDateString();
        return new Date(sTemp);
    };

    // Input string datetime
    // Output date result
    this.getTimeFromDateTime = function (datetime) {
        var arr = datetime.split(/[- :]/);
        var temp = new Date(arr[0], arr[1] - 1, arr[2], arr[3], arr[4], arr[5]);
        var sTemp = temp.getTime();
        return new Date(sTemp);
    };
    
    this.getDateTimeFromString = function(datetime){
        var arr = datetime.split(/[- :]/);
        var date = new Date(arr[0], arr[1] - 1, arr[2], arr[3], arr[4], arr[5]);
        return new Date(date);
    };
    this.getObjectOfArrByName = function (arr, name) {
        var result = null;
        arr.forEach(function (obj) {
            if (obj.name === $.trim(name)) {
                result = obj;
            }
        });
        return result;
    };
    this.goBackToLogin = function(currPageName, currParams){
        delete currParams['previousPage'];
        $rootScope.currentPage = {currentName: currPageName, params: currParams};
        $state.transitionTo(page_name.login);
    };


    this.compareAddressEdit = function (arrAddressOld, arrAddressNew) {
//        {'country':'', 'state':'', 'city':'', 'address':'', 'block_code':'','zip_code':'', 
//        'type':'','id':-1,'has_change': 0,'is_delete': 0}
        angular.forEach(arrAddressNew, function (addr_new) {
            var item = $filter('filter')(arrAddressOld, {id: addr_new.id})[0];
            if (item) {
                if (addr_new.is_delete !== '1' && addr_new.id !== '-1') {
                    var isEdit = false;
                    //compare country, state, city, address, block_code, zip_code, type
                    if (addr_new.country && item.country && addr_new.country !== item.country) {
                        isEdit = true;
                    }
                    if (addr_new.state && item.state && addr_new.state !== item.state) {
                        isEdit = true;
                    }
                    if (addr_new.city && item.city && addr_new.city !== item.city) {
                        isEdit = true;
                    }
                    if (addr_new.address && item.address && addr_new.address !== item.address) {
                        isEdit = true;
                    }
                    if (addr_new.block_code && item.block_code && addr_new.block_code !== item.block_code) {
                        isEdit = true;
                    }
                    if (addr_new.zip_code && item.zip_code && addr_new.zip_code !== item.zip_code) {
                        isEdit = true;
                    }
                    if (addr_new.type && item.type && addr_new.type !== item.type) {
                        isEdit = true;
                    }
                    //update is_change
                    if (isEdit) {
                        addr_new.has_change = '1';
                    }
                    else {
                        addr_new.has_change = '0';
                    }
                }

                delete item['cities'];
                delete item['states'];
            }
        });
    };

    this.turnOffIndicator = function ($scope) {
        $scope.$on("$ionicView.beforeLeave", function () {
            $rootScope.$broadcast('loading:show');
        });

        $scope.$on("$ionicView.enter", function () {
            $rootScope.$broadcast('loading:hide');
        });
    };
    
    this.isParseDataSuccess = function(parseResult){
        if (!parseResult.result) {
            //showAlertError(constant.language.mess_title_error, parseResult.obj.message);
            return false;
        }
        return true;
    };
    
    this.isHasPromotionApplied = function(){
        var cart = OrderCart.getCart();
        return cart.promotion ? true : false;
    };
    
    this.isAddressEmpty = function(address) {
        if(!address.country || (address.country && address.country.id > 0 )) 
            return false;
        if(!address.state || (address.state && address.state.id > 0)) 
            return false;
        if(address.address) 
            return false;
        if(address.zip_code) 
            return false;
        
        return true;
    };
        
    // Output: number with 0 before when < 10
    var parseNumber = this.parseNumber = function(number){
        if(0 < number && number < 10){
            return ('0' + number);
        }
        else {
            return number;
        }
    };
    
    var parseToFloat = this.parseToFloat = function(value){
        return parseFloat(value);
    };
    
    this.skip = function(){
        var dfd = $q.defer();
        GlobalUser.getUser().then(function (data) {
            if (data && emptyOrNull(data.user_id)) {
                $state.transitionTo(page_name.index);
            } else {
                $state.transitionTo(page_name.login);
            }
            dfd.resolve();
        });
        return dfd.promise;
    };
    
    this.getBase64FromImageUrl = function(url, callback) {
        var img = new Image();
        img.crossOrigin = 'Anonymous';
        img.onload = function(){
            var canvas = document.createElement('CANVAS');
            var ctx = canvas.getContext('2d');
            var dataURL;
            canvas.height = this.height;
            canvas.width = this.width;
            ctx.drawImage(this, 0, 0);
            dataURL = canvas.toDataURL("image/png");
            callback(dataURL);
            canvas = null; 
        };
        img.src = url;
    };
});